#include "model.h"
#include "clutility.h"
#include "machine.h"
#include <sstream>

AsynchronousModel::AsynchronousModel(Machine *parent, string model_definition) : Model(parent)
{
    queue_len = 8;
    neighbor = 8;
    loadSource();
}

AsynchronousModel::~AsynchronousModel()
{
    clReleaseKernel(init_kernel);
    clReleaseKernel(intermediate_kernel);
}

int AsynchronousModel::getCellSize()
{
    size_t event = sizeof(float)*2;
    size_t cell = sizeof(int)*4+event*queue_len+event;
    return cell;
}

void AsynchronousModel::loadSource()
{
    stringstream s;
    s << "-I./kernels/ -D NEIGHBOR=" << neighbor << " -D QUEUE_LEN=" << queue_len;
    string source = "__constant int2 neighbors[] = {-1,1, 1,1, -1,-1, 1,-1, 0,1, -1,0, 1,0, 0,-1, 0,0 };\n";
    source = "#include \"asyn.cl\"";
    build(source, s.str());

    init_kernel = clCreateKernel(program, "init", NULL);
    intermediate_kernel = clCreateKernel(program, "intermediateStep", NULL);
}

void AsynchronousModel::renderToTexture(CellSpace *pattern, int which, cl_mem tex)
{
    cl_int err_code;
    err_code = clSetKernelArg(render_kernel, 0, sizeof(cl_mem), pattern->getSpace());
    err_code |= clSetKernelArg(render_kernel, 1, sizeof(cl_mem), &tex);
    if(err_code){ cerr << "Nepodarilo sa nastavit argumenty "; clGetError(err_code); }
    parent->run(render_kernel, pattern->getWidth(), pattern->getHeight());
}

bool cmp16(float *a, float *b)
{
    bool ret = true;
    for(int i=0;i<16;i++)
        if(a[i]!=b[i])ret=false;
    return ret;
}

void AsynchronousModel::run(CellSpace *pattern, int which, int iteration)
{
    cl_int err_code;
    err_code = clSetKernelArg(kernel, 0, sizeof(cl_mem), pattern->getSpace());
    err_code |= clSetKernelArg(intermediate_kernel, 0, sizeof(cl_mem), pattern->getSpace());
    Variant f = pattern->getValue("current_time");
    if(err_code){ cerr << "Nepodarilo sa nastavit argumenty kernela "; clGetError(err_code); }

    for(int i=0;i<iteration;i++)
    {
        err_code = clSetKernelArg(kernel, 1, sizeof(float), &f);
        f.f += 100;
        err_code |= clSetKernelArg(kernel, 2, sizeof(float), &f);
        if(err_code)cerr << "Nepodarilo sa nastavit argument " << endl;
        parent->run(kernel, pattern->getWidth(), pattern->getHeight());
        parent->finish();
        //vypisem dubug informacie
        parent->run(intermediate_kernel, pattern->getWidth(), pattern->getHeight());
        parent->finish();
    }
    pattern->setValue("current_time", f);
}

/**
inicializuje data
@param pattern
*/
void AsynchronousModel::initialize(CellSpace *pattern, float *data)
{
    cl_int err_code;
    cl_mem buffer = parent->createBuffer(CL_MEM_READ_ONLY | CL_MEM_USE_HOST_PTR, sizeof(float)*pattern->getHeight()*pattern->getWidth(), data);
    if(!buffer){ cerr << "Nepodarilo sa vytvorit buffer pre inicializaciu modelu" << endl; return; }
    err_code = clSetKernelArg(init_kernel, 0, sizeof(cl_mem), pattern->getSpace());
    err_code|= clSetKernelArg(init_kernel, 1, sizeof(cl_mem), &buffer);
    if(err_code){ cerr << "Nepodarilo sa nastavit argument pre inicializacny kernel"; clGetError(err_code); return; }
    parent->run(init_kernel, pattern->getWidth(), pattern->getHeight());
    parent->finish();
    clReleaseMemObject(buffer);
}
