#include "cellspace.h"
#include "model.h"
#include "machine.h"
#include "string.h"
#include <stdlib.h>
/**
Vytvory stavovy priestor modelu
@param parent rodic ku ktoremu je dany stavovy priestor priradeny
@param w sirka v pocte buniek
@param h vyska v pocte buniek
*/
CellSpace::CellSpace(Model *parent, int w, int h)
{
    cl_int err_code;
    this->parent = parent;
    parent->addChild(this);
    which = 0;
    width = w;
    height = h;
    int size = parent->getCellSize()*w*h;
    char *data = new char[size];
    int *d = (int*)data;
    for(int i=0;i<size/4;i++)
    {
        d[i] = rand()%100;
        if(d[i]<10)d[i]=1;
        else d[i]=0;
    }
    //memset(data, 0, size);
    space = clCreateBuffer(parent->getParent()->getContext(), CL_MEM_READ_WRITE | CL_MEM_COPY_HOST_PTR, size, data, &err_code);
    delete data;
    if(err_code){ cerr << "Nepodarilo sa alokovat pamet"; clGetError(err_code); }
}

CellSpace::~CellSpace()
{
    parent->removeChild(this);
    clReleaseMemObject(space);
}

/**
Spusti simulaciu daneho pristoru
@param iteration pocet iteracii ktore sa maju vykonat
*/
void CellSpace::run(int iteration)
{
    parent->run(this, which, iteration);
    which=(which+iteration)&1;//binarna hodnota
}
/**
Vykresli do textury aktualny stav priestoru
@return pointer na pole pixmapy s formatom RGBA s rozmermy width height ktore vracia
@sa CellSpace::getWidth CellSpace::getHeight
*/
unsigned char* CellSpace::renderToTexture()
{
    cl_int err_code;
    unsigned char *data = new unsigned char[width*height*4];
    cl_mem buf = clCreateBuffer(parent->getParent()->getContext(), CL_MEM_WRITE_ONLY, width*height*4, NULL, &err_code);
    if(err_code){ cerr << "Nepodarilo sa vytvorit buffer pre texturu "; clGetError(err_code); }
    parent->renderToTexture(this, which, buf);
    parent->getParent()->copy(buf, data, width*height*4);
    //parent->getParent()->finish();
    clReleaseMemObject(buf);
    return data;
};

void CellSpace::setValue(string key, Variant v)
{
    values[key] = v;
}

Variant CellSpace::getValue(string key)
{
    return values[key];
}
