#include "glwidget.h"
#include <iostream>
#include <QPixmap>
#include <QImage>
#include <QMouseEvent>
#include "cellspace.h"
#include "model.h"
#include "machine.h"

/**
@param format format ktory sa ma pouzit pri vytvarani OpenGL okna. pozri Qt dokumentaciu ku QGLWidget
@param parent Qt rodic
@param space model ktory sa priradi aktualnemu oknu
*/
GLwidget::GLwidget(const QGLFormat &format, QWidget *parent, CellSpace *space) : QGLWidget(format, parent)
{
    pos_x = pos_y = 0;
    zoom = 1;
    this->space = space;
    if(space)connect(&timer, SIGNAL(timeout()), this, SLOT(runSimulation()));
}

GLwidget::~GLwidget()
{
    //if(space)delete space;
    //makeCurrent();
    //glDeleteTextures(1, &tex);
}
/**
Inicializacia OpenGL
*/
void GLwidget::initializeGL()
{
    //glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glGenTextures(1, &tex);
    glBindTexture(GL_TEXTURE_2D, tex);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glEnable(GL_TEXTURE_2D);
    if(!initialized)
    {
        std::cout << "Inicializujem OpenGL" << std::endl;
        std::cout << "\tVendor: \t" << glGetString(GL_VENDOR) << std::endl;
        std::cout << "\tRenderer:\t" << glGetString(GL_RENDERER) << std::endl;
        std::cout << "\tVersion:\t" << glGetString(GL_VERSION) << std::endl;
        //std::cout << "\tGLSL version:\t" << glGetString(GL_SHADING_LANGUAGE_VERSION) << std::endl;
        initialized = true;
    }
    if(space)
    {
        unsigned char *tex = space->renderToTexture();
        loadTexture(space->getWidth(), space->getHeight(), tex);
        delete [] tex;
    }
    //updateGL();
}
/**
Vykreslovanie OpenGL sceny
*/
void GLwidget::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex);
    glBegin(GL_QUADS);
    //glColor3f(1.0f, 0.0f, 0.0f);
    /*glTexCoord2d(0.0f, 0.0f);glVertex3f(-1.0f, -1.0f, 0.0f);
    glTexCoord2d(1.0f, 0.0f);glVertex3f( 1.0f, -1.0f, 0.0f);
    glTexCoord2d(1.0f, 1.0f);glVertex3f( 1.0f,  1.0f, 0.0f);
    glTexCoord2d(0.0f, 1.0f);glVertex3f(-1.0f,  1.0f, 0.0f);*/
    glTexCoord2d((float)pos_x/tex_w, (float)pos_y/tex_h);glVertex3f(-1.0f, -1.0f, 0.0f);
    glTexCoord2d((float)(pos_x+w*zoom)/tex_w, (float)pos_y/tex_h);glVertex3f( 1.0f, -1.0f, 0.0f);
    glTexCoord2d((float)(pos_x+w*zoom)/tex_w, (float)(pos_y+h*zoom)/tex_h);glVertex3f( 1.0f,  1.0f, 0.0f);
    glTexCoord2d((float)pos_x/tex_w, (float)(pos_y+h*zoom)/tex_h);glVertex3f(-1.0f,  1.0f, 0.0f);
    glEnd();

}

void GLwidget::resizeGL(int w,int h)
{
    glViewport(0, 0, w, h);
    this->w = w;
    this->h = h;
}
/**
Nacita texturu do OpenGL do akktualneho widgetu
@param w sirka textury
@param h vyska textury
@param texture pixelove data textury
*/
void GLwidget::loadTexture(int w, int h, unsigned char *texture)
{
    glBindTexture(GL_TEXTURE_2D, tex);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, texture);
    tex_w = w;
    tex_h = h;
}

void GLwidget::mouseMoveEvent(QMouseEvent *event)
{
    pos_x -= (event->x() - mouse_x)*zoom;
    pos_y += (event->y() - mouse_y)*zoom;
    mouse_x = event->x();
    mouse_y = event->y();
    updateGL();
}

void GLwidget::mousePressEvent(QMouseEvent *event)
{
    mouse_x = event->x();
    mouse_y = event->y();
}

void GLwidget::wheelEvent(QWheelEvent *event)
{

    if(event->delta() > 0)//zoom in
    {
        zoom *= 2;
        /*pos_x = pos_x + w/4;
        pos_y = pos_y + h/4;*/
    }
    else//zoom out
    {
        if(zoom <= 1/16.f)return;//nad sestnastnasobne zvetsenie nejdem
        zoom /= 2;
        /*pos_x = pos_x - w/2;
        pos_y = pos_y - h/2;*/
    }
    updateGL();
}

bool GLwidget::initialized = false;

void GLwidget::startSimulation()
{
    timer.start(1000);
}

void GLwidget::stopSimulation()
{
    timer.stop();
}

void GLwidget::runSimulation()
{
    space->run(1);
    space->getParent()->getParent()->finish();
    unsigned char *tex = space->renderToTexture();
    makeCurrent();
    loadTexture(space->getWidth(), space->getHeight(), tex);
    delete [] tex;
    updateGL();
}
