#define QUEUE_LEN 16
#define NEIGHBOR 9

#define COORD(c) (c.x+c.y*prop.size.y)
#define INT_CLAMP(i,m) (i>m ? 0 : i)

#include "types.cl"
#include "functions.cl"

/* vráti najmenší čas*/
_time get_next_time(int2 *coord, int *start, _cellspace_prop prop, __global _cell *cells)
{
    _time min;
    for(int i=0;i<NEIGHBOR;i++)
        min = fmin(min, cells[COORD(coord[i])].events[start[i]+1].x);
    return min;
}

/* posunie lokalne ukazovatele na poslednu poziciu ktora je <= ako cur_time */
void advance_in_queue(int2 *coord, int *start, int *end, _cellspace_prop prop, __global _cell *cells, _time cur_time)
{
    int next;
    for(int i=0;i<NEIGHBOR;i++)
    {
        next = start[i]==end[i] ? start[i] : start[i]+1;//priratat jednu poziciu ak nie sme na konci
        next = (next>=NEIGHBOR ? 0 : next);//ak prekrocime hranicu radu vratim sa na zaciatok
        start[i] = cells[COORD(coord[i])].events[next].x <= cur_time ? next : start[i];//nakoniec sa posuniem ak je cas mensi alebo rovny
    }
}

bool comp(_event a, _event b);
void transfer_func(int2 *coord, int *start, _cellspace_prop prop, __global _cell *cells);

/* */
__kernel void takeStep(_cellspace_prop prop, __global _cell *cells, _time current_time, _time time_window)
{
    int2 coord[NEIGHBOR];//suradnice jedno
    int current[NEIGHBOR];
    int ends[NEIGHBOR];
    int old_end;

    //prekopirujem si konce
    coord[0] = (int2)(get_global_id(0), get_global_id(1));
    current[0] = cells[COORD(coord[0])].start;
    ends[0] = old_end = cells[COORD(coord[0])].end;
    for(int i=1;i<NEIGHBOR;i++)
    {
        coord[i] = coord[0]+neighbors[i];
        coord[i].x = wrap(coord[i].x, prop.size.x); coord[i].y = wrap(coord[i].y, prop.size.y);
        current[i] = cells[COORD(coord[i])].start;
        ends[i] = cells[COORD(coord[i])].end;
    }
    mem_fence(CLK_GLOBAL_MEM_FENCE);
    _time next_time = current_time;
    while(1)
    {
        next_time = get_next_time(coord, current, prop, cells);//najdem minimalny cas z nasledujucich udalosti
        if(next_time >= current_time+time_window)break;//opustim casove okno ak som spracoval vsetky udalosti v okne
        advance_in_queue(coord, current, ends, prop, cells, next_time);//popus
        transfer_func(coord, current, prop, cells);
    }
}

__kernel void renderToTexture(_cellspace_prop prop, __global _cell *cells)
{
}

/*
nasledovne funkcie bude potrebne dynamicky generovat
*/

_event transfer_func(int2 *coord, int *start, _cellspace_prop prop, __global _cell *cells)
{
    _event new_event;
    /*[life-rule]
    rule : 1 100 { (0,0) = 1 and (truecount = 3 or truecount = 4) }
    rule : 1 100 { (0,0) = 0 and truecount = 2 }
    rule : 0 100 { t }*/
    if(cells[COORD(coord[0])].events[start[0]])new_event.y = 1;
    //if()
    //if(true){}
}

/* porovna dve udalosti ci maju rovnaky stav */
bool comp(_event a, _event b)
{
    return a.y == b.y ? true : false;
}
