#ifdef NEUMANN
__constant int neighbors[] = {
    0,1, -1,0, 1,0, 0,-1, 0,0 };
#define NEIGHBORS 5
#endif

#ifdef MOORE
__constant int neighbors[] = {
    -1,1, 1,1, -1,-1, 1,-1, 0, 1, -1,0, 1,0, 0,-1, 0,0 };
#define NEIGHBORS 9
#endif

int wrap(int x, int y)
{
    int r = (x < 0) ? x+y : x;
    return (r < y ? r : r-y);
}

int2 wrap2(int2 x, int2 y)
{
    return (int2)(wrap(x.x, y.x), wrap(x.y, y.y));
}

__kernel __attribute__((reqd_workgroup_size(16, 16, 1))) void takeStep(__global int *cells, int which)
{
    __local int loc_rule_tree[sizeof(rule_tree)/sizeof(int)];
    __local int2 loc_neighbors[NEIGHBORS];

    size_t lid = get_local_id(0)*get_local_size(1)+get_local_id(1);
    if(lid < sizeof(rule_tree)/sizeof(int))loc_rule_tree[lid] = rule_tree[lid];
    if(lid < NEIGHBORS)loc_neighbors[lid] = (int2)(neighbors[lid*2], neighbors[lid*2+1]);
    barrier(CLK_LOCAL_MEM_FENCE);

    size_t x = get_global_id(0);
    size_t y = get_global_id(1);
    size_t w = get_global_size(0);
    size_t h = get_global_size(1);
    int2 coord = (int2)(x, y);
    int2 sizes = (int2)(w, h);

    //if(x==0&&y==0)printf("lol %d\n", which);

    int node = sizeof(rule_tree)/sizeof(int) - NUM_STATES;
    int state;
    for(int i=0;i<NEIGHBORS;i++)
    {
        int2 c = wrap2(coord+loc_neighbors[i], sizes);
        state = cells[(c.y*w+c.x)*2+which];
        node = loc_rule_tree[node+state]*NUM_STATES;
        //if(x==255&&y==0)printf("%d %d  %d %d\n", state, node, c.x, c.y);
    }
    which = (which+1)&1;
    cells[(y*w+x)*2+which] = node/NUM_STATES;
}

__kernel void renderToTexture(__global int *cells, int which, __global unsigned int *tex)
{
    size_t x = get_global_id(0);
    size_t y = get_global_id(1);
    size_t w = get_global_size(0);
    int state = cells[(y*w+x)*2+which];
    uint c = colors[state*4]<<24 | colors[state*4+1]<<16 | colors[state*4+2]<<8 | colors[state*4+3];
    tex[y*w+x] = c;
}
