#ifndef _MACHINE_H_NOU_
#define _MACHINE_H_NOU_

#include <iostream>
#include <vector>
#include <CL/cl.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include "clutility.h"
#include "model.h"

using namespace std;

typedef vector<cl_device_id> cl_device_v;
typedef vector<cl_platform_id> cl_platform_v;
/**
Trieda zastresuje vytvorenie OpenCL kontextu v ktorom su spravovane vypoctove zariadenia
*/
class Machine
{
    private:
    cl_context context;
    cl_device_v device;
    vector<cl_command_queue> queues;
    vector<Model*> models;
    size_t local_work_size;
    bool gl_interoperability;
    bool deleting;
    void init(cl_context_properties *cl_properties, cl_device_v &devices);
    public:
    Machine(int platform_index = 0, cl_device_v devices = cl_device_v());
    Machine(cl_context_properties gl_context, cl_context_properties display, int platform_index = 0, cl_device_v devices = cl_device_v());
    ~Machine();
    static cl_platform_v getPlatforms();
    static cl_device_v getDevices(cl_platform_id platform_id, cl_device_type type = CL_DEVICE_TYPE_ALL);
    void run(cl_kernel kernel, size_t w, size_t h, int device = 0);
    void addChild(Model *child);
    void removeChild(Model *child);
    void finish();
    cl_event copy(cl_mem buf, void *data, size_t size, bool block = true);
    cl_event copy(void *data, cl_mem buf, size_t size, bool block = true);
    cl_event copy(cl_mem buf1, cl_mem buf2, size_t size, bool block = true);
    cl_context getContext();
    cl_mem createBuffer(cl_mem_flags flags, size_t size, void *ptr = 0);
    /** @return true ak je aktivna OpenGL/OpenCL interoperabilita */
    bool GLinteroprability(){ return gl_interoperability; };
    protected:
};

#endif
