#include "model.h"
#include "machine.h"

Model::Model(Machine *parent)
{
    this->parent = parent;
    parent->addChild(this);
    kernel = NULL;
    render_kernel = NULL;
    program = NULL;
}

Model::~Model()
{
    parent->removeChild(this);
    clReleaseKernel(kernel);
    clReleaseKernel(render_kernel);
    clReleaseProgram(program);
}
/**
Zaregistruje potomka.
@param child ukazovatel na potomka ktory sa ma pridat.
*/
void Model::addChild(CellSpace *child)
{
    patterns.push_back(child);
}
/**
Odstrani potomka zo zoznamu. Treba ho nasledne vymazat osobitne.
*/
void Model::removeChild(CellSpace *child)
{
    for(vector<CellSpace*>::iterator i = patterns.begin(); i != patterns.end();i++)
        if(*i == child)patterns.erase(i);
}

/*void Model::run(CellSpace *pattern, int which)
{

}*/
/**
Vytvori program pre model a kernel.
Zostvy program z predaneho zdrojoveho kodu. Ten musi definovat dva kernel.
takeStep a renderToTexture ktore su potom pouzivane.
@param source zdrojovy kod modelu
@param options volitelne volby kompilacie
*/
cl_int Model::build(string source, string options)
{
    cl_int err_code;
    const char *s = source.c_str();
    size_t len[] = { source.length() };
    program = clCreateProgramWithSource(parent->getContext(), 1, &s, len, &err_code);
    if(err_code){ cerr << "Nepodarilo sa vytvorit program "; clGetError(err_code); return err_code; }
    err_code = clBuildProgram(program, 0, NULL, options.c_str(), NULL, NULL);
    if(err_code){ cerr << "Nepodarilo sa skompilovat program " << endl << getBuildLog(program) << endl; return err_code; }
    kernel = clCreateKernel(program, "takeStep", &err_code);
    if(err_code){ cerr << "Nepodarilo sa vytvorit kernel "; clGetError(err_code); return err_code; }
    render_kernel = clCreateKernel(program, "renderToTexture", &err_code);
    if(err_code){ cerr << "Nepodarilo sa vytvorit kresliaci kernel "; clGetError(err_code); return err_code; }
    return err_code;
}
