#include "options.h"
#include <QCheckBox>
#include <QBitArray>
#include <QLabel>

Options::Options(QWidget *parent) : QDialog(parent)
{
    vertical_layout = new QVBoxLayout(this);
    //resize(400, 300);
    setWindowTitle(trUtf8("Nastavenia"));
    tabs = new QTabWidget(this);

    ok_cancel = new QDialogButtonBox(this);
    ok_cancel->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);

    QPushButton *o = ok_cancel->button(QDialogButtonBox::Ok);
    connect((QObject*)o, SIGNAL(pressed()), this, SLOT(accept()));
    o = ok_cancel->button(QDialogButtonBox::Cancel);
    connect((QObject*)o, SIGNAL(pressed()), this, SLOT(reject()));

    vertical_layout->addWidget(tabs);
    vertical_layout->addWidget(ok_cancel);
    createOpenCL();
    reject();//v reject() je nacitanie povodnych hodnout
}

Options::~Options()
{
    //dtor
}

void Options::accept()
{
    //TODO dorobit vsetky ostatne nastavenia
    settings.setValue("opencl/platform", platforms->currentIndex());

    QLayout *layout = platform_stack->currentWidget()->layout();
    int count = layout->count();
    QBitArray devices(count);
    for(int i=0;i<count;i++)
    {
        QCheckBox *box = dynamic_cast<QCheckBox*>(layout->itemAt(i)->widget());
        if(box && box->isChecked())devices.setBit(i);
    }
    settings.setValue("opencl/devices", devices);

    QDialog::accept();
}

void Options::reject()
{
    platforms->setCurrentIndex(settings.value("opencl/platform").toInt());

    QLayout *layout = platform_stack->widget(settings.value("opencl/platform", 0).toInt())->layout();
    int count = layout->count();
    QBitArray devices(count);
    devices = settings.value("opencl/devices", QBitArray(count)).toBitArray();
    devices.resize(count);
    for(int i=0;i<count;i++)
    {
        QCheckBox *box = dynamic_cast<QCheckBox*>(layout->itemAt(i)->widget());
        if(box)box->setChecked(devices[i]);
    }
    QDialog::reject();
}

void Options::createOpenCL()
{
    opencl = new QWidget(this);
    tabs->addTab(opencl, trUtf8("OpenCL"));
    platform_stack = new QStackedWidget(this);

    platforms = new QComboBox(this);
    cl_platform_v platform_ids;
    platform_ids = Machine::getPlatforms();

    QWidget *platform_widget;
    for(cl_platform_v::iterator i = platform_ids.begin(); i != platform_ids.end();i++)
    {
        platforms->addItem(getPlatformInfo(*i).c_str());
        platform_widget = new QWidget(platform_stack);
        platform_widget->setLayout(new QVBoxLayout(platform_widget));
        cl_device_v devices = Machine::getDevices(*i);
        for(cl_device_v::iterator o = devices.begin(); o != devices.end(); o++)
            platform_widget->layout()->addWidget(new QCheckBox(getDeviceInfoShort(*o).c_str()));
        platform_stack->addWidget(platform_widget);
    }

    opencl->setLayout(new QVBoxLayout(opencl));
    opencl->layout()->addWidget(platforms);
    opencl->layout()->addWidget(platform_stack);

    connect(platforms, SIGNAL(activated(int)), platform_stack, SLOT(setCurrentIndex(int)));
}

cl_device_v Options::getDevices()
{
    int platform_index = platforms->currentIndex();
    cl_device_v devices = Machine::getDevices(Machine::getPlatforms()[platform_index]);
    cl_device_v ret;
    QBitArray selected_devices = settings.value("opencl/devices").toBitArray();

    for(int i=0; i<selected_devices.size();i++)
        if(selected_devices[i])ret.push_back(devices[i]);
    return ret;
}
