#include "model.h"
#include "clutility.h"
#include "machine.h"

SynchronousModel::SynchronousModel(Machine *parent, string model_definition) : Model(parent)
{
    source = model_definition;
    loadSource();
}

void SynchronousModel::loadSource()
{
    string source = "__constant int rule_tree[] = {"
    "0,0,0,0,0,1,0,2,1,3,1,1,2,5,3,6,4,7,5,0,6,9,7,10,8,11,9,1,10,13,11,14,12,15,1,1,13,17,14,"
    "18,15,19,16,20,17,17,18,22,19,23,20,24,21,25,22,22,23,27,24,28,25,29,26,30};\n"
    "__constant uint colors[] = { 0, 0, 0, 0,255, 255, 255, 255 };\n #include \"synchr.cl\" ";
    build(source, "-I ./kernels/ -D MOORE -D NUM_STATES=2");
}

void SynchronousModel::run(CellSpace *pattern, int which, int iteration)
{
    cl_int err_code;
    err_code = clSetKernelArg(kernel, 0, sizeof(cl_mem), pattern->getSpace());
    err_code |= clSetKernelArg(kernel, 1, sizeof(int), &which);
    if(err_code){ cerr << "Nepodarilo sa nastavit argumenty kernela "; clGetError(err_code); }

    for(int i=0;i<iteration;i++)
    {
        err_code = clSetKernelArg(kernel, 1, sizeof(int), &which);
        if(err_code)cerr << "Nepodarilo sa nastavit argument " << endl;
        parent->run(kernel, pattern->getWidth(), pattern->getHeight());
        which = (which+1)&1;
    }
}

void SynchronousModel::renderToTexture(CellSpace *pattern, int which, cl_mem tex)
{
    cl_int err_code;
    err_code = clSetKernelArg(render_kernel, 0, sizeof(cl_mem), pattern->getSpace());
    err_code |= clSetKernelArg(render_kernel, 1, sizeof(int), &which);
    err_code |= clSetKernelArg(render_kernel, 2, sizeof(cl_mem), &tex);
    if(err_code){ cerr << "Nepodarilo sa nastavit argumenty "; clGetError(err_code); }
    parent->run(render_kernel, pattern->getWidth(), pattern->getHeight());
}
